@@ -1,47 +1,28 @@
+//! # Example
+//!
+//! ```
+//! use sliceslice::x86::DynamicAvx2Searcher;
+//!
+//! let searcher = unsafe { DynamicAvx2Searcher::new(b"ipsum") };
+//!
+//! assert!(unsafe {
+//!     searcher.search_in(b"Lorem ipsum dolor sit amet, consectetur adipiscing elit")
+//! });
+//!
+//! assert!(!unsafe {
+//!     searcher.search_in(b"foo bar baz qux quux quuz corge grault garply waldo fred")
+//! });
+//! ```
+
 #![allow(clippy::missing_safety_doc)]
 
-use crate::{bits, memcmp, MemchrSearcher, Needle};
+use crate::{MemchrSearcher, Needle, NeedleWithSize, Searcher, Vector, VectorHash};
 use seq_macro::seq;
 #[cfg(target_arch = "x86")]
 use std::arch::x86::*;
 #[cfg(target_arch = "x86_64")]
 use std::arch::x86_64::*;
 
-trait NeedleWithSize: Needle {
-    #[inline]
-    fn size(&self) -> usize {
-        if let Some(size) = Self::SIZE {
-            size
-        } else {
-            self.as_bytes().len()
-        }
-    }
-
-    #[inline]
-    fn is_empty(&self) -> bool {
-        self.size() == 0
-    }
-}
-
-impl<N: Needle + ?Sized> NeedleWithSize for N {}
-
-/// Represents an SIMD register type that is x86-specific (but could be used
-/// more generically) in order to share functionality between SSE2, AVX2 and
-/// possibly future implementations.
-trait Vector: Copy {
-    const LANES: usize;
-
-    unsafe fn set1_epi8(a: i8) -> Self;
-
-    unsafe fn loadu_si(a: *const u8) -> Self;
-
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self;
-
-    unsafe fn and_si(a: Self, b: Self) -> Self;
-
-    unsafe fn movemask_epi8(a: Self) -> i32;
-}
-
 #[derive(Clone, Copy)]
 #[repr(transparent)]
 #[allow(non_camel_case_types)]
@@ -49,35 +30,43 @@ struct __m16i(__m128i);
 
 impl Vector for __m16i {
     const LANES: usize = 2;
+    type Mask = Self;
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn set1_epi8(a: i8) -> Self {
-        __m16i(_mm_set1_epi8(a))
+    unsafe fn splat(a: u8) -> Self {
+        __m16i(_mm_set1_epi8(a as i8))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn loadu_si(a: *const u8) -> Self {
+    unsafe fn load(a: *const u8) -> Self {
         __m16i(_mm_set1_epi16(std::ptr::read_unaligned(a as *const i16)))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self {
+    unsafe fn lanes_eq(a: Self, b: Self) -> Self {
         __m16i(_mm_cmpeq_epi8(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn and_si(a: Self, b: Self) -> Self {
+    unsafe fn bitwise_and(a: Self, b: Self) -> Self {
         __m16i(_mm_and_si128(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn movemask_epi8(a: Self) -> i32 {
-        _mm_movemask_epi8(a.0) & 0x3
+    unsafe fn to_bitmask(a: Self) -> u32 {
+        std::mem::transmute(_mm_movemask_epi8(a.0) & 0x3)
+    }
+}
+
+impl From<__m128i> for __m16i {
+    #[inline]
+    fn from(vector: __m128i) -> Self {
+        Self(vector)
     }
 }
 
@@ -88,35 +77,43 @@ struct __m32i(__m128i);
 
 impl Vector for __m32i {
     const LANES: usize = 4;
+    type Mask = Self;
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn set1_epi8(a: i8) -> Self {
-        __m32i(_mm_set1_epi8(a))
+    unsafe fn splat(a: u8) -> Self {
+        __m32i(_mm_set1_epi8(a as i8))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn loadu_si(a: *const u8) -> Self {
+    unsafe fn load(a: *const u8) -> Self {
         __m32i(_mm_set1_epi32(std::ptr::read_unaligned(a as *const i32)))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self {
+    unsafe fn lanes_eq(a: Self, b: Self) -> Self {
         __m32i(_mm_cmpeq_epi8(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn and_si(a: Self, b: Self) -> Self {
+    unsafe fn bitwise_and(a: Self, b: Self) -> Self {
         __m32i(_mm_and_si128(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn movemask_epi8(a: Self) -> i32 {
-        _mm_movemask_epi8(a.0) & 0xF
+    unsafe fn to_bitmask(a: Self) -> u32 {
+        std::mem::transmute(_mm_movemask_epi8(a.0) & 0xF)
+    }
+}
+
+impl From<__m128i> for __m32i {
+    #[inline]
+    fn from(vector: __m128i) -> Self {
+        Self(vector)
     }
 }
 
@@ -127,152 +124,113 @@ struct __m64i(__m128i);
 
 impl Vector for __m64i {
     const LANES: usize = 8;
+    type Mask = Self;
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn set1_epi8(a: i8) -> Self {
-        __m64i(_mm_set1_epi8(a))
+    unsafe fn splat(a: u8) -> Self {
+        __m64i(_mm_set1_epi8(a as i8))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn loadu_si(a: *const u8) -> Self {
+    unsafe fn load(a: *const u8) -> Self {
         __m64i(_mm_set1_epi64x(std::ptr::read_unaligned(a as *const i64)))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self {
+    unsafe fn lanes_eq(a: Self, b: Self) -> Self {
         __m64i(_mm_cmpeq_epi8(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn and_si(a: Self, b: Self) -> Self {
+    unsafe fn bitwise_and(a: Self, b: Self) -> Self {
         __m64i(_mm_and_si128(a.0, b.0))
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn movemask_epi8(a: Self) -> i32 {
-        _mm_movemask_epi8(a.0) & 0xFF
+    unsafe fn to_bitmask(a: Self) -> u32 {
+        std::mem::transmute(_mm_movemask_epi8(a.0) & 0xFF)
+    }
+}
+
+impl From<__m128i> for __m64i {
+    #[inline]
+    fn from(vector: __m128i) -> Self {
+        Self(vector)
     }
 }
 
 impl Vector for __m128i {
     const LANES: usize = 16;
+    type Mask = Self;
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn set1_epi8(a: i8) -> Self {
-        _mm_set1_epi8(a)
+    unsafe fn splat(a: u8) -> Self {
+        _mm_set1_epi8(a as i8)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn loadu_si(a: *const u8) -> Self {
+    unsafe fn load(a: *const u8) -> Self {
         _mm_loadu_si128(a as *const Self)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self {
+    unsafe fn lanes_eq(a: Self, b: Self) -> Self {
         _mm_cmpeq_epi8(a, b)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn and_si(a: Self, b: Self) -> Self {
+    unsafe fn bitwise_and(a: Self, b: Self) -> Self {
         _mm_and_si128(a, b)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn movemask_epi8(a: Self) -> i32 {
-        _mm_movemask_epi8(a)
+    unsafe fn to_bitmask(a: Self) -> u32 {
+        std::mem::transmute(_mm_movemask_epi8(a))
     }
 }
 
 impl Vector for __m256i {
     const LANES: usize = 32;
+    type Mask = Self;
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn set1_epi8(a: i8) -> Self {
-        _mm256_set1_epi8(a)
+    unsafe fn splat(a: u8) -> Self {
+        _mm256_set1_epi8(a as i8)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn loadu_si(a: *const u8) -> Self {
+    unsafe fn load(a: *const u8) -> Self {
         _mm256_loadu_si256(a as *const Self)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn cmpeq_epi8(a: Self, b: Self) -> Self {
+    unsafe fn lanes_eq(a: Self, b: Self) -> Self {
         _mm256_cmpeq_epi8(a, b)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn and_si(a: Self, b: Self) -> Self {
+    unsafe fn bitwise_and(a: Self, b: Self) -> Self {
         _mm256_and_si256(a, b)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
-    unsafe fn movemask_epi8(a: Self) -> i32 {
-        _mm256_movemask_epi8(a)
-    }
-}
-
-/// Hash of the first and "last" bytes in the needle for use with the SIMD
-/// algorithm implemented by `Avx2Searcher::vector_search_in`. As explained, any
-/// byte can be chosen to represent the "last" byte of the hash to prevent
-/// worst-case attacks.
-struct VectorHash<V: Vector> {
-    first: V,
-    last: V,
-}
-
-impl<V: Vector> VectorHash<V> {
-    #[target_feature(enable = "avx2")]
-    unsafe fn new(first: u8, last: u8) -> Self {
-        Self {
-            first: Vector::set1_epi8(first as i8),
-            last: Vector::set1_epi8(last as i8),
-        }
-    }
-}
-
-impl From<&VectorHash<__m128i>> for VectorHash<__m64i> {
-    #[inline]
-    fn from(hash: &VectorHash<__m128i>) -> Self {
-        Self {
-            first: __m64i(hash.first),
-            last: __m64i(hash.last),
-        }
-    }
-}
-
-impl From<&VectorHash<__m128i>> for VectorHash<__m32i> {
-    #[inline]
-    fn from(hash: &VectorHash<__m128i>) -> Self {
-        Self {
-            first: __m32i(hash.first),
-            last: __m32i(hash.last),
-        }
-    }
-}
-
-impl From<&VectorHash<__m128i>> for VectorHash<__m16i> {
-    #[inline]
-    fn from(hash: &VectorHash<__m128i>) -> Self {
-        Self {
-            first: __m16i(hash.first),
-            last: __m16i(hash.last),
-        }
+    unsafe fn to_bitmask(a: Self) -> u32 {
+        std::mem::transmute(_mm256_movemask_epi8(a))
     }
 }
 
@@ -357,121 +315,37 @@ impl<N: Needle> Avx2Searcher<N> {
         }
     }
 
-    #[inline]
-    #[target_feature(enable = "avx2")]
-    unsafe fn vector_search_in_chunk<V: Vector>(
-        &self,
-        haystack: &[u8],
-        hash: &VectorHash<V>,
-        start: *const u8,
-        mask: i32,
-    ) -> bool {
-        let first = Vector::loadu_si(start);
-        let last = Vector::loadu_si(start.add(self.position));
-
-        let eq_first = Vector::cmpeq_epi8(hash.first, first);
-        let eq_last = Vector::cmpeq_epi8(hash.last, last);
-
-        let eq = Vector::and_si(eq_first, eq_last);
-        let mut eq = (Vector::movemask_epi8(eq) & mask) as u32;
-
-        let start = start as usize - haystack.as_ptr() as usize;
-        let chunk = haystack.as_ptr().add(start + 1);
-        let needle = self.needle.as_bytes().as_ptr().add(1);
-
-        while eq != 0 {
-            let chunk = chunk.add(eq.trailing_zeros() as usize);
-            let equal = match N::SIZE {
-                Some(0) => unreachable!(),
-                Some(1) => memcmp::specialized::<0>(chunk, needle),
-                Some(2) => memcmp::specialized::<1>(chunk, needle),
-                Some(3) => memcmp::specialized::<2>(chunk, needle),
-                Some(4) => memcmp::specialized::<3>(chunk, needle),
-                Some(5) => memcmp::specialized::<4>(chunk, needle),
-                Some(6) => memcmp::specialized::<5>(chunk, needle),
-                Some(7) => memcmp::specialized::<6>(chunk, needle),
-                Some(8) => memcmp::specialized::<7>(chunk, needle),
-                Some(9) => memcmp::specialized::<8>(chunk, needle),
-                Some(10) => memcmp::specialized::<9>(chunk, needle),
-                Some(11) => memcmp::specialized::<10>(chunk, needle),
-                Some(12) => memcmp::specialized::<11>(chunk, needle),
-                Some(13) => memcmp::specialized::<12>(chunk, needle),
-                Some(14) => memcmp::specialized::<13>(chunk, needle),
-                Some(15) => memcmp::specialized::<14>(chunk, needle),
-                Some(16) => memcmp::specialized::<15>(chunk, needle),
-                _ => memcmp::generic(chunk, needle, self.needle.size() - 1),
-            };
-            if equal {
-                return true;
-            }
-
-            eq = bits::clear_leftmost_set(eq);
-        }
-
-        false
-    }
-
-    #[inline]
-    #[target_feature(enable = "avx2")]
-    unsafe fn vector_search_in<V: Vector>(
-        &self,
-        haystack: &[u8],
-        end: usize,
-        hash: &VectorHash<V>,
-    ) -> bool {
-        debug_assert!(haystack.len() >= self.needle.size());
-
-        let mut chunks = haystack[..end].chunks_exact(V::LANES);
-        while let Some(chunk) = chunks.next() {
-            if self.vector_search_in_chunk(haystack, hash, chunk.as_ptr(), -1) {
-                return true;
-            }
-        }
-
-        let remainder = chunks.remainder().len();
-        if remainder > 0 {
-            let start = haystack.as_ptr().add(end - V::LANES);
-            let mask = -1 << (V::LANES - remainder);
-
-            if self.vector_search_in_chunk(haystack, hash, start, mask) {
-                return true;
-            }
-        }
-
-        false
-    }
-
     #[inline]
     #[target_feature(enable = "avx2")]
     unsafe fn sse2_2_search_in(&self, haystack: &[u8], end: usize) -> bool {
         let hash = VectorHash::<__m16i>::from(&self.sse2_hash);
-        self.vector_search_in(haystack, end, &hash)
+        self.vector_search_in_avx2_version(haystack, end, &hash)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
     unsafe fn sse2_4_search_in(&self, haystack: &[u8], end: usize) -> bool {
         let hash = VectorHash::<__m32i>::from(&self.sse2_hash);
-        self.vector_search_in(haystack, end, &hash)
+        self.vector_search_in_avx2_version(haystack, end, &hash)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
     unsafe fn sse2_8_search_in(&self, haystack: &[u8], end: usize) -> bool {
         let hash = VectorHash::<__m64i>::from(&self.sse2_hash);
-        self.vector_search_in(haystack, end, &hash)
+        self.vector_search_in_avx2_version(haystack, end, &hash)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
     unsafe fn sse2_16_search_in(&self, haystack: &[u8], end: usize) -> bool {
-        self.vector_search_in(haystack, end, &self.sse2_hash)
+        self.vector_search_in_avx2_version(haystack, end, &self.sse2_hash)
     }
 
     #[inline]
     #[target_feature(enable = "avx2")]
     unsafe fn avx2_search_in(&self, haystack: &[u8], end: usize) -> bool {
-        self.vector_search_in(haystack, end, &self.avx2_hash)
+        self.vector_search_in_avx2_version(haystack, end, &self.avx2_hash)
     }
 
     /// Inlined version of `search_in` for hot call sites.
@@ -506,6 +380,18 @@ impl<N: Needle> Avx2Searcher<N> {
     }
 }
 
+impl<N: Needle> Searcher<N> for Avx2Searcher<N> {
+    #[inline(always)]
+    fn needle(&self) -> &N {
+        &self.needle
+    }
+
+    #[inline(always)]
+    fn position(&self) -> usize {
+        self.position
+    }
+}
+
 /// Single-substring searcher based on `Avx2Searcher` but with dynamic algorithm
 /// selection.
 ///
@@ -555,7 +441,7 @@ pub enum DynamicAvx2Searcher<N: Needle> {
 
 macro_rules! array {
     ($c:ident, $S:literal) => [seq!(N in 0..$S {
-            [ #( $c#N, )* ]
+            [ #( $c #N, )* ]
     })];
 }
 
@@ -642,154 +528,6 @@ mod tests {
     use super::{Avx2Searcher, DynamicAvx2Searcher};
     use crate::Needle;
 
-    fn search(haystack: &[u8], needle: &[u8]) -> bool {
-        let search =
-            |position| unsafe { Avx2Searcher::with_position(needle, position).search_in(haystack) };
-
-        let dynamic_search = |position| unsafe {
-            DynamicAvx2Searcher::with_position(needle.to_owned().into_boxed_slice(), position)
-                .search_in(haystack)
-        };
-
-        let result = search(0);
-        for position in 1..needle.len() {
-            assert_eq!(search(position), result);
-            assert_eq!(dynamic_search(position), result);
-        }
-
-        result
-    }
-
-    #[test]
-    fn search_same() {
-        assert!(search(b"x", b"x"));
-
-        assert!(search(b"xy", b"xy"));
-
-        assert!(search(b"foo", b"foo"));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus"
-        ));
-    }
-
-    #[test]
-    fn search_different() {
-        assert!(!search(b"x", b"y"));
-
-        assert!(!search(b"xy", b"xz"));
-
-        assert!(!search(b"bar", b"foo"));
-
-        assert!(!search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"foo"
-        ));
-
-        assert!(!search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"foo"
-        ));
-
-        assert!(!search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"foo bar baz qux quux quuz corge grault garply waldo fred plugh xyzzy thud"
-        ));
-    }
-
-    #[test]
-    fn search_prefix() {
-        assert!(search(b"xy", b"x"));
-
-        assert!(search(b"foobar", b"foo"));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"Lorem"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"Lorem"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit"
-        ));
-    }
-
-    #[test]
-    fn search_suffix() {
-        assert!(search(b"xy", b"y"));
-
-        assert!(search(b"foobar", b"bar"));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"elit"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"purus"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"Aliquam iaculis fringilla mi, nec aliquet purus"
-        ));
-    }
-
-    #[test]
-    fn search_multiple() {
-        assert!(search(b"xx", b"x"));
-
-        assert!(search(b"xyxy", b"xy"));
-
-        assert!(search(b"foobarfoo", b"foo"));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"it"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"conse"
-        ));
-    }
-
-    #[test]
-    fn search_middle() {
-        assert!(search(b"xyz", b"y"));
-
-        assert!(search(b"wxyz", b"xy"));
-
-        assert!(search(b"foobarfoo", b"bar"));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit",
-            b"consectetur"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"orci"
-        ));
-
-        assert!(search(
-            b"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas commodo posuere orci a consectetur. Ut mattis turpis ut auctor consequat. Aliquam iaculis fringilla mi, nec aliquet purus",
-            b"Maecenas commodo posuere orci a consectetur"
-        ));
-    }
-
     #[test]
     #[should_panic]
     fn avx2_invalid_position() {
@@ -817,7 +555,7 @@ mod tests {
             const SIZE: Option<usize> = Some(2);
 
             fn as_bytes(&self) -> &[u8] {
-                &self.0
+                self.0
             }
         }
 
@@ -845,4 +583,28 @@ mod tests {
         assert_eq!(size_of::<DynamicAvx2Searcher::<[u8; 16]>>(), 160);
         assert_eq!(size_of::<DynamicAvx2Searcher::<Box<[u8]>>>(), 160);
     }
+
+    impl crate::tests::TestSearcher for Avx2Searcher<&[u8]> {
+        fn with_position(needle: &'static [u8], position: usize) -> Avx2Searcher<&[u8]> {
+            unsafe { Avx2Searcher::with_position(needle, position) }
+        }
+
+        fn search_in(&self, haystack: &[u8]) -> bool {
+            unsafe { Avx2Searcher::search_in(self, haystack) }
+        }
+    }
+
+    crate::generate_tests!(avx2_searcher, Avx2Searcher);
+
+    impl crate::tests::TestSearcher for DynamicAvx2Searcher<&[u8]> {
+        fn with_position(needle: &'static [u8], position: usize) -> DynamicAvx2Searcher<&[u8]> {
+            unsafe { DynamicAvx2Searcher::with_position(needle, position) }
+        }
+
+        fn search_in(&self, haystack: &[u8]) -> bool {
+            unsafe { DynamicAvx2Searcher::search_in(self, haystack) }
+        }
+    }
+
+    crate::generate_tests!(dynamic_avx2_searcher, DynamicAvx2Searcher);
 }
